*************************************************************************************************************************;
* This program will generate Wald, Peto's and score confidence intervals under the Cox model;
* By Gang, Jan 10, 2016
*************************************************************************************************************************;

*************************************************************************************************************************;
* Macro to generate Wald, Peto's and score confidence intervals under the Cox model;
* Data:     the dataset for time to event analysis;
* Time:     the time to the event or when the patient was censored;
* Status:    the numeric code corresponding to event (1) or for censoring (0);
* Stratum:  the numeric code of stratification factor, if there is no stratification variable, 
            leave it empty;
* Treatment:the factor in the model, e.g., treatment;
* Alpha:    the type I error;
* Accuracy: the accuray to achieve for score CI;
* Result:   the estimates and CI with Wald, Score test, and Peto methods, p-values will also be included;  
*************************************************************************************************************************;
%macro HazardRatio(Data = Data, Time = Time, Status = Status, Stratum = , Treatment = trt, Alpha = 0.05, Accuracy = 0.001, Result = HR_CI);

%if %LENGTH(&Stratum.) = 0 %then %do;
    %let Stratum = HRTMP_Stratum;
data sb_orig;
    keep &Stratum. &Time. &Status. &Treatment.  Status_SAS;
    set &Data.;
    if &Status. = 1 then Status_SAS = 0;
    else Status_SAS = 1;
    &Stratum. = 1; 
run;
%end; 
%else %do;
data sb_orig;
    keep &Stratum. &Time. &Status. &Treatment.  Status_SAS;
    set &Data.;
    if &Status. = 1 then Status_SAS = 0;
    else Status_SAS = 1;
%end; 

ods output ParameterEstimates = sb_Wald;
proc phreg data= sb_orig;
    model &Time.*Status_SAS(1) = &Treatment. /ties=Breslow;
    strata &Stratum.;
run;
ods output close;

data sb_bisection;
    keep name Lower Upper Chisq;
    set sb_Wald;
    name = "Lower Bound";
    Lower = Estimate - 2.96*StdErr;
    Upper = Estimate;
    Chisq = CINV((1-&Alpha.), 1); 
    output;
    name = "Upper Bound";
    Lower = Estimate;
    Upper = Estimate + 2.96*StdErr;
    Chisq = CINV((1-&Alpha.), 1); 
    output;
run;

/*Obtain Wald estimate*/
data &Result.;
    length Method $8.;
    retain Method Estimate StdErr Lower Upper ChiSq P_value;
    keep Method Estimate StdErr Lower Upper ChiSq P_value;
    set sb_Wald;
    Method = "Wald";
    Lower = Estimate - probit(1-&Alpha./2)*StdErr;
    Upper = Estimate + probit(1-&Alpha./2)*StdErr;
    P_value = ProbChiSq;
run;

/*Obtain the estimate from Score test result*/
/*Step 1: Obtain the estimate*/
data sb_Wald_estimate;
    keep Estimate StdErr;
    set sb_Wald;
run;

/*Step 2: Obtain the CI*/
/*Get the lower bound*/
data sb_lower;
    set sb_bisection;
    where name = "Lower Bound";
    call symput('Lower',Lower);
    call symput('Upper',Upper);
    call symput('Chisq',Chisq);
run;
*%score_bisection(Lower = &Lower., Upper = &Upper., Threshold = &Chisq., Accuracy = &Accuracy., Data = &Data., Time = &Time., Status = &Status., Stratum = &Stratum., X = &Treatment.);
%score_bisection(Lower = &Lower., Upper = &Upper., Threshold = &Chisq., Accuracy = &Accuracy., Data = sb_orig, Time = &Time., Status = &Status., Stratum = &Stratum., X = &Treatment.);

data sb_lower_result;
    keep Lower;
    set Sbi_Result;
    Lower = Value;
run;

/*Get the upper bound*/
data sb_upper;
    set sb_bisection;
    where name = "Upper Bound";
    call symput('Lower',Lower);
    call symput('Upper',Upper);
    call symput('Chisq',Chisq);
run;
*%score_bisection(Lower = &Lower., Upper = &Upper., Threshold = &Chisq., Accuracy = &Accuracy., Data = &Data., Time = &Time., Status = &Status., Stratum = &Stratum., X = &Treatment.);
%score_bisection(Lower = &Lower., Upper = &Upper., Threshold = &Chisq., Accuracy = &Accuracy., Data = sb_orig, Time = &Time., Status = &Status., Stratum = &Stratum., X = &Treatment.);

data sb_upper_result;
    keep Upper;
    set Sbi_Result;
    Upper = Value;
run;

/*Step 3: Obtain the p-value*/
*%score_cox(Data = &Data., Time = &Time., Status = &Status., Stratum = &Stratum., X = &Treatment., Beta = 0);
%score_cox(Data = sb_orig, Time = &Time., Status = &Status., Stratum = &Stratum., X = &Treatment., Beta = 0);
data sb_score_pvalue;
    keep ChiSq P_value;
    set SC_function;
    ChiSq = SC;
    P_value = 1 - probchi(ChiSq, 1);
run;

/*Combine the result*/
data sb_score;
    keep Method Estimate StdErr Lower Upper ChiSq P_value;
    merge sb_Wald_estimate sb_lower_result sb_upper_result sb_score_pvalue;
    Method = "Score";
run;

/*Peto-Estimates*/
data sb_peto;
    retain Method Estimate StdErr Lower Upper ChiSq P_value;
    keep Method Estimate StdErr Lower Upper ChiSq P_value;
    set SC_function;
    Method = "Peto";
    Estimate = U/I;
    StdErr = 1/sqrt(I);
    Lower = Estimate - probit(1-&Alpha./2)/sqrt(I);
    Upper = Estimate + probit(1-&Alpha./2)/sqrt(I);
    ChiSq = SC;
    P_value = 1 - probchi(ChiSq, 1);
run;

data &Result.;
    set &Result. sb_score sb_peto;
    Lower = round(Lower,&Accuracy.);
    Upper = round(Upper,&Accuracy.);
    P_value = round(P_value,0.0001);
run;

proc datasets noprint;
    delete sb_orig sb_Wald sb_Wald_estimate sb_bisection Sbi_result sb_lower sb_lower_result sb_upper sb_upper_result sb_score sb_score_pvalue sb_peto SC_function;
run;
quit;
%mend HazardRatio;

*************************************************************************************************************************;
* Example to use;
*************************************************************************************************************************;
*%HazardRatio(Data = temp, Time = Time, Status = Status, Stratum = Stratum, Treatment = trt, Alpha = 0.05, Accuracy = 0.001);




/*The following two macros are supplemental functions for the main one*/

*************************************************************************************************************************;
* Macro to generate the value of score test under the Cox model given beta;
* Data:     the dataset for time to event analysis;
* Time:     the time to the event or when the patient was censored;
* Status:    the numeric code corresponding to event (1) or for censoring (0);
* Stratum:  the numeric code of stratification factor, if there is no stratification variable, 
            then specify one var with the same value of 1;
* X:        the factor in the model, e.g., treatment;
* Beta:     the given value for Beta;
* Result:   the estimates of U, I and score function;
*************************************************************************************************************************;
%macro score_cox(Data = Data, Time = Time, Status = Status, Stratum = Stratum, X = X, Beta = Beta, Result = SC_function);
data sc_orig;
    keep &Stratum. &Time. &Status. &X. X_EXP X_EXP_MX Event_Count;
    set &Data.;
    X_EXP = exp(&X.*&Beta.);
    X_EXP_MX = X_EXP*&X.;

    /*Event_Count: to count the number of events occurred at specific time point*/
    if &Status. = 1 then Event_Count = 1;
    else Event_Count = 0;
run;

/*Calculate the cumulative information at each time point*/
proc sort data = sc_orig;
    by &Stratum. &Time.;
run;
proc sql;
    create table sc_orig_time as select
        &Stratum., 
        &Time.,
        count(*) as Count_D,
        sum(Event_Count) as Count_D_Event,
        sum(X_EXP) as Time_ED,
        sum(X_EXP_MX) as Time_EN
    from sc_orig
    group by &Stratum., &Time.
    order by &Stratum., &Time.; 
quit;

proc sort data = sc_orig_time;
    by &Stratum. descending &Time.;
run;
data sc_orig_time_cum;
    retain Count_R Time_ED_Cum Time_EN_Cum 0;
    set sc_orig_time;
    by &Stratum.;
    if first.&Stratum. then do;
        Count_R = 0;
        Time_ED_Cum = 0;
        Time_EN_Cum = 0;   
    end;
        Count_R = Count_R + Count_D;
        Time_ED_Cum = Time_ED_Cum + Time_ED;
        Time_EN_Cum = Time_EN_Cum + Time_EN;   
run;
data sc_orig_time_cum_final;
    keep &Stratum. &Time. Count_D Count_D_Event Count_R Time_ED_Cum Time_EN_Cum;
    set sc_orig_time_cum;
run;

/*Obtain the data to calculate the statistic for score test*/
proc sort data = sc_orig_time_cum_final;
    by &Stratum. &Time.;
run;
data sc_orig_calculation;
    merge sc_orig sc_orig_time_cum_final;
    by &Stratum. &Time.;
    Time_E = Time_EN_Cum/Time_ED_Cum;
    U_Subject = &Status.*(&X.-Time_E);
    if (Count_R = 1) then I_Subject = 0;
    else I_Subject = &Status.*(1-Time_E)*Time_E*(Count_R-Count_D_Event)/(Count_R-1);
run;

/*Calculate the score test*/
proc sql;
    create table &Result. as select
        sum(U_Subject) as U,
        sum(I_Subject) as I 
    from sc_orig_calculation;
quit;
data &Result.;
    set &Result.;
    SC = U*U/I;
run;

proc datasets noprint;
    delete sc_orig sc_orig_time sc_orig_time_cum sc_orig_time_cum_final sc_orig_calculation;
run;
quit;
%mend score_cox;

*************************************************************************************************************************;
* The bisection method is used
* Lower:    the lower bound for the investigation interval;
* Upper:    the upper bound for the investigation interval;
* Threshold:the value to identify the interval; 
* Accuracy: the accuray to achieve for score CI;
* Data:     the dataset for time to event analysis;
* Time:     the time to the event or when the patient was censored;
* Status:    the numeric code corresponding to event (1) or for censoring (0);
* Stratum:  the numeric code of stratification factor, if there is no stratification variable, 
            then specify one var with the same value of 1;
* X:        the factor in the model, e.g., treatment;
* Result:   the estimate with bisection method;
*************************************************************************************************************************;
%macro score_bisection(Lower =, Upper =, Threshold = , Accuracy = 0.001, Data = Data, Time = Time, Status = Status, Stratum = Stratum, X = X, Result = Sbi_Result);

%let diff = %SYSEVALF(&Upper. - &Lower.);
%let middle = %SYSEVALF((&Upper. + &Lower.)/2);

%do %while (%SYSEVALF(&diff. > &Accuracy.));

    %score_cox(Data = &Data., Time = &Time., Status = &Status., Stratum = &Stratum., X = &X., Beta = &Lower.);
    data sbi_value;
        set SC_function;
        call symput('value_L',SC);
    run;
    %score_cox(Data = &Data., Time = &Time., Status = &Status., Stratum = &Stratum., X = &X., Beta = &middle.);
    data sbi_value;
        set SC_function;
        call symput('value_M',SC);
    run;
    %let direction = %SYSEVALF((&value_L. - &Threshold.)*(&value_M. - &Threshold.));

    %if (%SYSEVALF(&direction.>0)) %then %let Lower = &middle.;
    %else %let Upper = &middle.;

    %let diff = %SYSEVALF(&Upper. - &Lower.);
    %let middle = %SYSEVALF((&Upper. + &Lower.)/2);
%end;

data &Result.;
    Lower = &Lower.;
    Upper = &Upper.;
    Value = round(&Lower.,&Accuracy.);
run;

proc datasets noprint;
    delete SC_function sbi_value;
run;
quit;
%mend score_bisection;
